/**
 * Copyright 2019 IBM Corp. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an 'AS IS' BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

const log4js = require('log4js');
const path = require('path');
const fs = require('fs');

const logger = log4js.getLogger('users - cloudant');
logger.level = 'debug';

const cloudantConfigPath = path.join(__dirname, './config/cloudant-credentials.json');
const cloudantConfig = require(cloudantConfigPath);
const configPath = path.join(__dirname, './config/config.json');
const config = require(configPath);

const cloudant = require('@cloudant/cloudant')(cloudantConfig, (err) => {
  if (err) {
    logger.error(`Failed to initialize Cloudant: ${err.message}`);
  }
  logger.debug('Cloudant init success.');
  return 0;
});

/**
 * Uploads a design document if the specified database has a design document
 * @param {string} dbName Name of the database
 */
function uploadDesignDoc(dbName, designDoc) {
  const designDocPath = (path.join(__dirname, `./config/${designDoc}`));
  fs.readFile(designDocPath, (err, data) => {
    if (err) {
      logger.error(`Failed to read document due to ${err.stack ? err.stack : err}`);
    } else {
      const document = JSON.parse(data);
      // insert the document
      cloudant.use(dbName).insert({ _id: document._id, document }, (error) => {
        if (error) {
          logger.error(`Failed to insert design document: ${document} due to ${error.stack ? error.stack : error}`);
        }
      });
    }
  });
}

/**
 * Creates the specified database through cloudant
 * @param {string} dbName Name of the Database
 */
async function createDB(dbName, designDoc) {
  await cloudant.db.create(dbName, async (error) => {
    if (error) {
      logger.error(`Failed to create ${dbName} due to ${error.stack ? error.stack : error}`);
    }
    logger.debug(`Created ${dbName} database`);
    // Specify it as the database to use
    cloudant.use(dbName);
    if (designDoc !== undefined) {
      await uploadDesignDoc(dbName, designDoc);
      logger.debug(`Design doc for ${dbName} inserted successfully.`);
    }
    logger.debug(`End Script >>> database ${dbName}`);
  });
}

/**
 * Deletes the specified database through cloudant
 * @param {string} dbName Name of the Database
 */
function deleteDB(dbName) {
  logger.debug(`Deleting database: ${dbName}`);
  return cloudant.db.destroy(dbName);
}

function main() {
  const selectedOption = process.argv[2];
  logger.debug(`Starting Cloudant script to recreate database with organization ${selectedOption}`);
  const org = selectedOption;

  const { databases } = config[org];

  databases.forEach((db) => {
    const dbName = db.name;
    logger.debug(`Start Script >>> database: ${dbName}`);
    logger.debug(`Fetched database ${dbName} from config file`);

    cloudant.db.get(dbName, async (err) => {
      // Check for error
      if (err) {
        // Database doesn't exist
        if (err.error === 'not_found') {
          logger.debug(`No ${dbName} found, creating ${dbName}`);
          await createDB(dbName, db.designDoc);
        }
      } else {
        // Database exists
        logger.debug(`Database: ${dbName} already exists.`);
        cloudant.use(dbName); // Specify it as the database to use
        deleteDB(dbName).then(() => {
          logger.debug(`Deleted ${dbName} database`);
          createDB(dbName, db.designDoc);
        }).catch((error) => {
          logger.error(`Failed to delete ${dbName} database due to ${error.stack ? error.stack : error}`);
        });
      }
    });
  });
}

main();
