/**
 * Copyright 2019 IBM Corp. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an 'AS IS' BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

const log4js = require('log4js');
const path = require('path');
const { CouchDBWallet, X509WalletMixin } = require('fabric-network');
const CertificateManagerWallet = require('@blockchainlabs/ibm-certificate-manager-wallet');
const fabricCAUtil = require('./helper/util.js');

const logger = log4js.getLogger('users - fabric');
logger.level = 'debug';
const usersConfigPath = path.join(__dirname, './config/identity-config.json');
const userList = require(usersConfigPath);

/**
 * Grabs the arguments passed into the command line for script
 * returns the args as an object
 */
function getArgs() {
  const args = {};
  process.argv
    .slice(2, process.argv.length)
    .forEach((arg) => {
      // long arg
      if (arg.slice(0, 2) === '--') {
        const formattedArg = arg.split('=');
        const argFlag = formattedArg[0].slice(2, formattedArg[0].length);
        const argValue = formattedArg.length > 1 ? formattedArg[1] : true;
        args[argFlag] = argValue;
      }
    });
  return args;
}

/**
 * Registers specified user through Fabric CA
 * @param {string} user Name of User
 */
async function registerUser(user) {
  logger.debug('Registering User in Fabric CA...');
  await fabricCAUtil.userRegister(user.user, user.pw, user.affiliation, user.role,
    user.attrs);
}

/**
 * Enroll user in Hyperledger Fabric and store certificate in persistent store
 * @param {string} user Name of user
 * @param {string} persistent Name of store
 */
async function enrollUser(user, persistent) {
  logger.debug('Enrolling User in Fabric CA...  new user');
  logger.debug('Storing User Certificate in Persistent Store...');
  const enrollInfo = await fabricCAUtil.userEnroll();
  enrollInfo.key = enrollInfo.key.toBytes(); // Change key to bytes
  /* eslint-disable global-require */
  let wallet;
  if (typeof (persistent) !== 'undefined' && persistent === 'cloudant') {
    const cloudantConfig = require('../fabric/config/cloudant-credentials.json');
    wallet = new CouchDBWallet(cloudantConfig);
    logger.debug('wallet initialized with Cloudant');
  } else if (typeof (persistent) === 'undefined' || persistent === 'certificate-manager') {
    const certManagerConfig = require('../fabric/config/certificate-manager-credentials.json');
    wallet = new CertificateManagerWallet(certManagerConfig);
    logger.debug('wallet initialized with IBM Certificate Manager');
  } else {
    logger.debug('Incorrect Usage of script. Refer README for more details');
    return;
  }
  /* eslint-enable global-require */
  // Add certificate and private key to wallet
  wallet.import(user.user, X509WalletMixin.createIdentity(user.org, enrollInfo.certificate,
    enrollInfo.key));
}

function main() {
  const args = getArgs();
  logger.debug(args);

  const { persistent } = args;
  logger.info(`Starting Fabric user enrollment and registration with persistent store: ${persistent || 'certificate-manager'}`);

  const { option } = args;
  logger.debug(`Selected Option: ${option || 'register & enroll'}`);

  const networkConfig = path.join(__dirname, './config/connection-profile.json');
  logger.debug(`Network config file: ${networkConfig}`);

  fabricCAUtil.init(networkConfig);
  logger.debug(`Number of Users in config file: ${userList.users.length}`);

  const { users } = userList;

  if (option === 'register') {
    users.forEach(async (user) => {
      logger.debug('Calling register user');
      await registerUser(user);
    });
  } else if (option === 'enroll') {
    users.forEach(async (user) => {
      await logger.debug('Calling enroll user');
      enrollUser(user, persistent);
    });
  } else { // default if none passed, do the registration & enroll
    users.forEach(async (user) => {
      logger.debug('Calling register user');
      await registerUser(user);
      logger.debug('Calling enroll user');
      await enrollUser(user, persistent);
    });
  }
}

main();
