/**
 * Copyright 2019 IBM Corp. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an 'AS IS' BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

const log4js = require('log4js');
const FabricCAServices = require('fabric-ca-client');
const FabricClient = require('fabric-client');
const fs = require('fs');

/**
 * Util object
 */
const util = {};

/**
 * Set up logging
 */
const logger = log4js.getLogger('helpers - util');
logger.level = 'debug';

/**
 * Initialize util with path of connection profile
 */
let connectionProfile;
let ccpValues;

util.init = (connProfilePath) => {
  connectionProfile = connProfilePath;
  // add network config file to fabric ca service and get orgs and CAs fields
  logger.debug('load connection profile');

  const ccpJSON = fs.readFileSync(connectionProfile, 'utf8');
  const ccp = JSON.parse(ccpJSON);
  logger.debug('connection profile loaded');

  const clientOrg = ccp.client.organization;
  const CaIP = ccp.organizations[clientOrg].certificateAuthorities[0];
  const { enrollId } = ccp.certificateAuthorities[CaIP].registrar[0];
  const { enrollSecret } = ccp.certificateAuthorities[CaIP].registrar[0];

  logger.debug(`Client Org: ${clientOrg}`);
  const org = ccp.organizations[clientOrg];
  logger.debug(`Org: ${JSON.stringify(org, 2, null)}`);
  const orgCAKey = org.certificateAuthorities[0];
  logger.debug(`Org CA Key:  ${orgCAKey}`);
  const caURL = ccp.certificateAuthorities[orgCAKey].url;
  logger.debug(`Org CA URL: ${caURL}`);
  const { caName } = ccp.certificateAuthorities[orgCAKey];
  logger.debug(`Org CA Name: ${caName}`);

  // store the values needed for the rest of the util functions
  ccpValues = {
    enrollId, enrollSecret, caURL, caName,
  };
};

/**
 * Enroll given user with given org Fabric CA
 */
util.userEnroll = () => new Promise(((resolve, reject) => {
  // enroll user with certificate authority for orgName
  const tlsOptions = {
    trustedRoots: [],
    verify: false,
  };
  const caService = new FabricCAServices(ccpValues.caURL, tlsOptions, ccpValues.caName);
  const req = {
    enrollmentID: ccpValues.enrollId,
    enrollmentSecret: ccpValues.enrollSecret,
    maxEnrollments: -1,

  };
  logger.debug(`Enrollment Request: \n${JSON.stringify(req, null, 2)}`);
  caService.enroll(req).then(
    (enrollment) => {
      logger.debug('enrolling');
      return resolve(enrollment);
    },
    (err) => {
      logger.debug(err);
      logger.debug(`err: ${err}`);
      return reject(err);
    },
  );
}));

/**
 * Enroll given user with given org Fabric CA
 */
util.userRegister = (user, userPw,
  affiliation, role, attrs) => {
  logger.debug(`Registering user: ${user}`);
  return new Promise(((resolve, reject) => {
    const fabricClient = new FabricClient();
    fabricClient.loadFromConfig(connectionProfile);
    FabricClient.newDefaultKeyValueStore({
      path: '.',
    }).then((stateStore) => {
      // assign the store to the fabric client
      fabricClient.setStateStore(stateStore);
      const cryptoSuite = FabricClient.newCryptoSuite();
      // use the same location for the state store (where the users' certificate are kept)
      // and the crypto store (where the users' keys are kept)
      const cryptoStore = FabricClient.newCryptoKeyStore({
        path: '.',
      });
      cryptoSuite.setCryptoKeyStore(cryptoStore);
      fabricClient.setCryptoSuite(cryptoSuite);
      return fabricClient.setUserContext({
        username: ccpValues.enrollId,
        password: ccpValues.enrollSecret,
      });
    }).then((registrarUser) => {
      // enroll user with certificate authority for orgName
      const tlsOptions = {
        trustedRoots: [],
        verify: false,
      };
      const caService = new FabricCAServices(ccpValues.caURL, tlsOptions, ccpValues.caName);

      const req = {
        enrollmentID: user,
        affiliation,
        role,
        enrollmentSecret: userPw,
        attrs,
        maxEnrollments: -1,
      };

      caService.register(req, registrarUser).then(
        (enrollment) => {
          logger.debug('enrolling...');
          return resolve(enrollment);
        },
      ).catch((err) => {
        logger.error(`err ${err}`);
        return reject(err);
      });
    });
  }));
};

module.exports = util;
