/**
 * Copyright 2019 IBM Corp. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an 'AS IS' BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

const request = require('request-promise');
const Q = require('q');
const path = require('path');
const log4js = require('log4js');

const logger = log4js.getLogger('users - app-user-registration');
logger.level = 'debug';

const appIDConfigPath = path.join(__dirname, './config', process.argv[2]);
const appIDconfig = require(appIDConfigPath);
logger.debug(`App ID config file: ${JSON.stringify(appIDconfig, null, 2)}`);

const usersConfigPath = path.join(__dirname, './config', process.argv[3]);
const usersConfig = require(usersConfigPath);
logger.debug(`User Details config file: ${JSON.stringify(usersConfig, null, 3)}`);

const selectedOption = process.argv[4];
logger.debug(`Selected Option: ${selectedOption}`);

/**
 * Generic HTTP call
 * @param {*} headers
 * @param {*} reqBody
 * @param {*} method
 * @param {*} url
 * @param {*} deferred
 * @param {*} isjson
 */
async function handleRequest(headers, reqBody, method, url, deferred, isjson) {
  logger.debug(`${method} request at: ${url}`);

  const options = {
    url,
    method,
    body: reqBody,
    headers,
    json: isjson,
    resolveWithFullResponse: true,
  };
  request(options)
    .then((response) => {
      if (response.statusCode === 204 && options.method === 'DELETE') {
        logger.debug(response.statusCode, 'Success');
      } else {
        logger.debug(response.statusCode, response.statusMessage);
      }
      return deferred.resolve(response.body);
    })
    .catch((err) => {
      logger.error(err.message);
    });
}


/**
 * Creates a user in cloud directory
 */
async function createUsersInCloudDirectory() {
  const deferred = Q.defer();
  const { users } = usersConfig;
  const accessToken = appIDconfig.authKey;
  const method = 'POST';
  const url = `${appIDconfig.managementUrl}/cloud_directory/Users`;
  const headers = {
    'Content-Type': 'application/json',
    Authorization: `Bearer ${accessToken}`,
  };

  users.forEach((user) => {
    const reqBody = user;
    logger.debug(`Request: ${JSON.stringify(reqBody, null, 2)}`);
    handleRequest(headers, reqBody, method, url, deferred, true);
  });
  return deferred.promise;
}

/**
 * Sign in the user to App ID
 */
async function signInUsersToAppId() {
  const deferred = Q.defer();
  const { users } = usersConfig;
  const method = 'POST';
  const url = `${appIDconfig.oauthServerUrl}/token/`;
  logger.debug(Buffer.from(`${appIDconfig.clientId}:${appIDconfig.secret}`).toString('base64'));
  const headers = {
    'Content-Type': 'application/json',
    Authorization: `Basic ${Buffer.from(`${appIDconfig.clientId}:${appIDconfig.secret}`).toString('base64')}`,
  };
  users.forEach(async (user) => {
    const reqBody = `{"username": "${user.userName}", "password": "${user.password}", "grant_type": "password"}`;
    handleRequest(headers, reqBody, method, url, deferred, false);
  });
  return deferred.promise;
}

/**
 * Get users profile attributes using users email address
 * @param  user
 */
async function getUserIdFromAppId(email) {
  const deferred = Q.defer();
  const accessToken = appIDconfig.authKey;
  const method = 'GET';
  // Use first and only email id
  const url = `${appIDconfig.managementUrl}/users/?email=${email}`;
  const headers = {
    'Content-Type': 'application/json',
    Authorization: `Bearer ${accessToken}`,
  };
  handleRequest(headers, null, method, url, deferred);
  return deferred.promise;
}

/**
 * Get users profile and returns the cloud directory id using users App ID userID
 * @param  userId
 */
async function getCloudDirectoryId(userId) {
  const deferred = Q.defer();
  const accessToken = appIDconfig.authKey;
  const method = 'GET';
  // Use first and only email id
  const url = `${appIDconfig.managementUrl}/users/${userId}/profile`;
  const headers = {
    'Content-Type': 'application/json',
    Authorization: `Bearer ${accessToken}`,
  };
  const reqBody = null;
  handleRequest(headers, reqBody, method, url, deferred);
  return deferred.promise;
}

/**
 * Removes user from cloud directory
 */
async function deleteUserFromCloudDirectory(userId) {
  const deferred = Q.defer();
  const accessToken = appIDconfig.authKey;
  const method = 'DELETE';
  const url = `${appIDconfig.managementUrl}/cloud_directory/Users/${userId}`;
  const headers = {
    'Content-Type': 'application/json',
    Authorization: `Bearer ${accessToken}`,
  };
  handleRequest(headers, null, method, url, deferred);
  return deferred.promise;
}

/**
 * Updates user profile attributes in App ID
 * No need to create user in App ID as once it is signed in it is already activated
 */
async function updateUsersProfileInAppId() {
  const deferred = Q.defer();
  const { users } = usersConfig;
  users.forEach(async (user) => {
    const email = user.emails[0].value;
    const response = await getUserIdFromAppId(email);
    logger.debug(`Response: ${response}`);
    const appIdUserDetails = JSON.parse(response);
    const accessToken = appIDconfig.authKey;
    const method = 'PUT';
    const headers = {
      'Content-Type': 'application/json',
      Authorization: `Bearer ${accessToken}`,
    };
    logger.debug('User: ', user);
    const url = `${appIDconfig.managementUrl}/users/${appIdUserDetails.users[0].id}/profile/`;
    const reqBody = JSON.stringify(user.profile);
    logger.debug(`Request: ${reqBody}`);
    handleRequest(headers, reqBody, method, url, deferred);
  });
  return deferred.promise;
}

/**
 * Removes user from App ID registry
 */
async function deleteUsersProfileInAppId() {
  const deferred = Q.defer();
  const { users } = usersConfig;

  users.forEach(async (user) => {
    const email = user.emails[0].value;
    const appIdUserId = await getUserIdFromAppId(email);
    logger.debug('App ID User ID:', appIdUserId);
    const appIdUserDetails = JSON.parse(appIdUserId);
    const accessToken = appIDconfig.authKey;
    if (appIdUserDetails.users < 1 || appIdUserDetails.users === undefined) {
      logger.debug('no App ID user id found.');
    } else {
      const cloudDirectoryProfile = await getCloudDirectoryId(appIdUserDetails.users[0].id);
      const cloudDirectoryProfileDetails = JSON.parse(cloudDirectoryProfile);
      await deleteUserFromCloudDirectory(cloudDirectoryProfileDetails.identities[0].id);
      const url = `${appIDconfig.managementUrl}/users/${appIdUserDetails.users[0].id}`;
      const method = 'DELETE';
      const headers = {
        'Content-Type': 'application/json',
        Authorization: `Bearer ${accessToken}`,
      };
      handleRequest(headers, null, method, url, deferred, true);
    }
  });
  return deferred.promise;
}

async function main() {
  logger.info('Start Executing Job');
  switch (selectedOption) {
    case '-create':
      // Step 1 Create users in cloud directory
      await createUsersInCloudDirectory();

      // Step 2 Sign in users to App ID
      await signInUsersToAppId();

      // Step 3 Update users profile attributes in App ID
      await updateUsersProfileInAppId();
      break;
    case '-update':
      // Step 1 Sign in users to App ID to get user's internal resource id
      await signInUsersToAppId();

      // Step 2 Update user's profile attributes in App ID
      await updateUsersProfileInAppId();
      break;
    case '-delete':
      // Step 1 Delete users from app id directory
      await deleteUsersProfileInAppId();
      break;
    default:
      logger.info('Invalid Options Selected! Valid options are `-create` to create users, `-update` to update user attributes and `-delete` to delete users.');
      break;
  }
}

main();
